ad_library {

	Library for Mash Up : Yahoo Weather

    @author Hamilton Chua (ham@solutiongrove.com)
    @creation-date 2006-03-20
}

namespace eval mashup::weather { }

# ******* WEATHER ***********

ad_proc -public mashup::weather::url {
	-yw_code
	{-temp_unit "c"}
} {
	Generate the url from the yahoo code and temp unit.
} {
	return "http://xml.weather.yahoo.com/forecastrss?p=$yw_code&u=$temp_unit"
}

ad_proc -public mashup::weather::new {
	-user_id
	-yw_code
	{-temp_unit "c"}
} {
	Create mashup weather record.
	Return the id of the new record
} {
	set id [db_nextval "mashup_weather_id"]
	db_dml "insertweather" "insert into mashup_weather (id, user_id, yw_code, temp_unit) values (:id,:user_id,:yw_code,:temp_unit)"
	return $id	
}

ad_proc -public mashup::weather::del {
	-user_id
	-id
} {
	Delete mashup weather record.
} {
	db_dml "delweather" "delete from mashup_weather where user_id=:user_id and id=:id"
}

ad_proc -public mashup::weather::tryget {
	{-id ""}
	{-yw_code ""}
	{-temp_unit "c"}
} {
	Try to retrieve the weather feed
	Return 0 if NOT successful,
	Returns the page part of the array which contains the feed.
} {
	# create a header to pass along with the http get
 	set headers [ns_set create]
 	ns_set put $headers "Referer" [parameter::get_from_package_key -package_key "news-aggregator" -parameter "referrer"]

	if { [exists_and_not_null id] } {
		set feed_url [mashup::weather::url -yw_code [db_string "getcode" "select yw_code from mashup_weather where id=:id"]]
	} else {
		set feed_url [mashup::weather::url -yw_code $yw_code]
	}
	
	# attempt to get the feed
	if { [catch { array set f [ad_httpget -url $feed_url -headers $headers] } error ] } {
        	ns_log Debug "MASHUP:weather::tryget: Couldn't retrieve $feed_url, error = $error"
        	return 0
    	}

    	if { ![string equal 200 $f(status)] } {
		ns_log Debug "MASHUP:weather::tryget: httpget didn't return 200 but $f(status)"
		return 0
    	}

	return $f(page)
}

ad_proc -public mashup::weather::parserss {
	-feed_content
} {
	Receives the feed content from mashup::weahter::tryget
	Attempts to parse the feed into an array that we can use
	later on  a page to display the values
	Returns 0 if parse fails.
	Example Usage :
		array set item [mashup::weather::parserss -feed_content [mashup::weather::tryget -yw_code $yw_code]]
		set title $item(title)
		set desc $item(description)
} {
	array set result [feed_parser::parse_feed -xml $feed_content]
	if { $result(status) eq "error" } {
        	ns_log Debug "MASHUP:weather::parserss: Couldn't parse feed, error = $result(error)"
		return 0
	}
	# ns_log Notice "HAM: $result(items)"
	foreach item $result(items) {
		array set oneitem $item
		# ns_log Notice "HAM: $oneitem(title)"
	}
	# array set oneitem $result(items)	
	return [array get oneitem]
}


